@@ -4,9 +4,9 @@ from pma_python import core, pma
 
 __version__ = pma.__version__
 
-_pma_casecollections_json = {}
-_pma_projects_json = {} 
-_pma_session_json = {}
+_pma_casecollections_json = {}	# reserved for future use
+_pma_projects_json = {} 		# reserved for future use
+_pma_session_json = {}			# reserved for future use
 
 def get_version_info(pmacontrolURL):
 	"""
@@ -21,7 +21,7 @@ def get_version_info(pmacontrolURL):
 		return None		
 	return r.json()
 
-def get_sessions(pmacontrolURL, pmacoreSessionID):
+def _pma_get_sessions(pmacontrolURL, pmacoreSessionID):
 	"""
 	Retrieve a list of currently defined training sessions in PMA.control.
 	"""
@@ -34,25 +34,108 @@ def get_sessions(pmacontrolURL, pmacoreSessionID):
 		return None		
 	return r.json()
 
-def get_session_ids(pmacontrolURL, pmacoreSessionID):
+def _pma_format_session_properly(sess):
+	"""
+	Helper method to convert a JSON representation of a PMA.control training session to a proper Python-esque structure
+	"""
+	sess_data = {
+		"Id": sess["Id"],
+		"Title": sess["Title"],
+		"LogoPath": sess["LogoPath"],
+		"StartsOn": sess["StartsOn"],
+		"EndsOn": sess["EndsOn"],
+		"ProjectId": sess["ModuleId"],
+		"State": sess["State"],
+		"CaseCollections": {}
+	}
+	for coll in sess["CaseCollections"]:
+		sess_data["CaseCollections"][coll["Id"]] = coll["Title"]
+
+	return sess_data
+
+def get_sessions(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID):
 	"""
 	Retrieve a dictionary with currently defined training sessions in PMA.control.
 	The resulting dictionary use the session's identifier as the dictionary key, and 
 	therefore this method is easier to quickly retrieve and represent session-related data. 
 	However, this method returns less verbose data than get_sessions()
 	"""
-	full_sessions = get_sessions(pmacontrolURL, pmacoreSessionID)
+	full_sessions = _pma_get_sessions(pmacontrolURL, pmacoreSessionID)
 	new_session_dict = {}
 	for sess in full_sessions:
-		sess_data = {
-			"LogoPath": sess["LogoPath"],
-			"StartsOn": sess["StartsOn"],
-			"EndsOn": sess["EndsOn"],
-			"ModuleId": sess["ModuleId"],
-			"State": sess["State"]
-		}
-		new_session_dict[sess["Id"]] = sess_data
+		if (pmacontrolProjectID is None) or (pmacontrolProjectID == sess["ModuleId"]):				
+			new_session_dict[sess["Id"]] = _pma_format_session_properly(sess)
 	return new_session_dict
+
+def get_sessions_for_participant(pmacontrolURL, pmacoreUsername, pmacoreSessionID):
+	full_sessions = _pma_get_sessions(pmacontrolURL, pmacoreSessionID)
+	new_session_dict = {}
+	for sess in full_sessions:
+		for part in sess["Participants"]:
+			if (part["User"].lower() == pmacoreUsername.lower()):
+				s = _pma_format_session_properly(sess)
+				s["Role"] = part["Role"]
+				new_session_dict[sess["Id"]] = s
+
+	return new_session_dict
+
+def get_session_participants(pmacontrolURL, pmacontrolSessionID, pmacoreSessionID):
+	"""
+	Extract the participants in a particular session
+	"""
+	full_sessions = _pma_get_sessions(pmacontrolURL, pmacoreSessionID)
+	user_dict = {}
+	for sess in full_sessions:
+		if sess["Id"] == pmacontrolSessionID:
+			for part in sess["Participants"]:
+				user_dict[part["User"]] = part["Role"]
+			return user_dict
+
+	return None
+
+def is_participant_in_session(pmacontrolURL, pmacoreUsername, pmacontrolSessionID, pmacoreSessionID):
+	"""
+	Check to see if a specific user participates in a specific session
+	"""
+	all_parts = get_session_participants(pmacontrolURL, pmacontrolSessionID, pmacoreSessionID)
+	return pmacoreUsername in all_parts
+	
+def get_session_titles(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID):
+	"""
+	Retrieve sessions (possibly filtered by project ID), titles only
+	"""
+	try:
+		return list(get_session_titles_dict(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID).values())
+	except Exception as e:
+		return None		
+	
+def get_session_titles_dict(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID):
+	"""
+	Retrieve (training) sessions (possibly filtered by project ID), return a dictionary of session IDs and titles
+	"""
+	dct = {}
+	all = _pma_get_sessions(pmacontrolURL, pmacoreSessionID)
+	for sess in all:
+		if pmacontrolProjectID == None:
+			dct[sess["Id"]] = sess["Title"]
+		elif pmacontrolProjectID == sess["ModuleId"]:
+			dct[sess["Id"]] = sess["Title"]
+
+	return dct
+
+	
+def search_session(pmacontrolURL, titleSubstring, pmacoreSessionID):
+	"""
+	Return the first (training) session that has titleSubstring as part of its string; search is case insensitive
+	"""
+	all = _pma_get_sessions(pmacontrolURL, pmacoreSessionID)
+
+	for el in all:
+		if titleSubstring.lower() in el['Title'].lower():
+			# summarize session-related information so that it makes sense
+			return _pma_format_session_properly(el)
+
+	return None
 	
 def _pma_get_case_collections(pmacontrolURL, pmacoreSessionID):
 	"""
@@ -62,16 +145,13 @@ def _pma_get_case_collections(pmacontrolURL, pmacoreSessionID):
 	global _pma_casecollections_json
 	
 	url = pma._pma_join(pmacontrolURL, "api/CaseCollections?sessionID=" + pma._pma_q(pmacoreSessionID))
-	if not (url in _pma_casecollections_json):
-		print (url)
-		try:
-			headers = {'Accept': 'application/json'}
-			r = requests.get(url, headers=headers)
-			_pma_casecollections_json[url] = r.json()
-		except Exception as e:
-			return None		
-
-	return _pma_casecollections_json[url]
+	print (url)
+	try:
+		headers = {'Accept': 'application/json'}
+		r = requests.get(url, headers=headers)
+		return r.json()
+	except Exception as e:
+		return None		
 
 def get_case_collection_titles(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID):
 	"""
@@ -125,16 +205,13 @@ def _pma_get_projects(pmacontrolURL, pmacoreSessionID):
 	global _pma_projects_json
 	
 	url = pma._pma_join(pmacontrolURL, "api/Projects?sessionID=" + pma._pma_q(pmacoreSessionID))
-	if not (url in _pma_projects_json):
-		print (url)
-		try:
-			headers = {'Accept': 'application/json'}
-			r = requests.get(url, headers=headers)
-		except Exception as e:
-			return None		
-		_pma_projects_json[url] = r.json()
-		
-	return _pma_projects_json[url]
+	print (url)
+	try:
+		headers = {'Accept': 'application/json'}
+		r = requests.get(url, headers=headers)
+		return r.json()
+	except Exception as e:
+		return None		
 
 def get_project_titles(pmacontrolURL, pmacoreSessionID):
 	"""
